@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef TIMELINENOTESRENDERPASS_H
-#define TIMELINENOTESRENDERPASS_H
+#pragma once
 
 #include "timelineabstractrenderer.h"
 #include <QSGMaterial>
@@ -38,13 +37,10 @@ public:
 
     State *update(const TimelineAbstractRenderer *renderer, const TimelineRenderState *parentState,
                   State *oldState, int firstIndex, int lastIndex, bool stateChanged,
-                  qreal spacing) const;
+                  float spacing) const;
 
 private:
     TimelineNotesRenderPass();
 };
 
 } // namespace Timeline
-
-#endif // TIMELINENOTESRENDERPASS_H
-
