@@ -68,58 +68,24 @@ const QLatin1String InstallFailedPermissionModelDowngrade("INSTALL_FAILED_PERMIS
 const QLatin1String InstallFailedVersionDowngrade("INSTALL_FAILED_VERSION_DOWNGRADE");
 const Core::Id AndroidDeployQtStep::Id("Qt4ProjectManager.AndroidDeployQtStep");
 
-//////////////////
+
 // AndroidDeployQtStepFactory
-/////////////////
 
-
-AndroidDeployQtStepFactory::AndroidDeployQtStepFactory(QObject *parent)
-    : IBuildStepFactory(parent)
+AndroidDeployQtStepFactory::AndroidDeployQtStepFactory()
 {
+    registerStep<AndroidDeployQtStep>(AndroidDeployQtStep::Id);
+    setSupportedStepList(ProjectExplorer::Constants::BUILDSTEPS_DEPLOY);
+    setSupportedDeviceType(Constants::ANDROID_DEVICE_TYPE);
+    setRepeatable(false);
+    setDisplayName(tr("Deploy to Android device or emulator"));
 }
 
-QList<BuildStepInfo> AndroidDeployQtStepFactory::availableSteps(BuildStepList *parent) const
-{
-    if (parent->id() != ProjectExplorer::Constants::BUILDSTEPS_DEPLOY
-            || !AndroidManager::supportsAndroid(parent->target())
-            || parent->contains(AndroidDeployQtStep::Id))
-        return {};
-
-    return {{AndroidDeployQtStep::Id, tr("Deploy to Android device or emulator")}};
-}
-
-ProjectExplorer::BuildStep *AndroidDeployQtStepFactory::create(ProjectExplorer::BuildStepList *parent, Core::Id id)
-{
-    Q_UNUSED(id);
-    return new AndroidDeployQtStep(parent);
-}
-
-ProjectExplorer::BuildStep *AndroidDeployQtStepFactory::clone(ProjectExplorer::BuildStepList *parent, ProjectExplorer::BuildStep *product)
-{
-    return new AndroidDeployQtStep(parent, static_cast<AndroidDeployQtStep *>(product));
-}
-
-//////////////////
 // AndroidDeployQtStep
-/////////////////
 
 AndroidDeployQtStep::AndroidDeployQtStep(ProjectExplorer::BuildStepList *parent)
     : ProjectExplorer::BuildStep(parent, Id)
-{
-    ctor();
-}
-
-AndroidDeployQtStep::AndroidDeployQtStep(ProjectExplorer::BuildStepList *parent,
-    AndroidDeployQtStep *other)
-    : ProjectExplorer::BuildStep(parent, other)
-{
-    ctor();
-}
-
-void AndroidDeployQtStep::ctor()
 {
     m_uninstallPreviousPackage = QtSupport::QtKitInformation::qtVersion(target()->kit())->qtVersion() < QtSupport::QtVersionNumber(5, 4, 0);
-    m_uninstallPreviousPackageRun = false;
 
     //: AndroidDeployQtStep default display name
     setDefaultDisplayName(tr("Deploy to Android device"));
@@ -161,12 +127,9 @@ bool AndroidDeployQtStep::init(QList<const BuildStep *> &earlierSteps)
     }
 
     int deviceAPILevel = AndroidManager::minimumSDK(target());
-    AndroidConfigurations::Options options = AndroidConfigurations::None;
-    if (androidBuildApkStep->deployAction() == AndroidBuildApkStep::DebugDeployment)
-        options = AndroidConfigurations::FilterAndroid5;
     AndroidDeviceInfo info = earlierDeviceInfo(earlierSteps, Id);
     if (!info.isValid()) {
-        info = AndroidConfigurations::showDeviceDialog(project(), deviceAPILevel, m_targetArch, options);
+        info = AndroidConfigurations::showDeviceDialog(project(), deviceAPILevel, m_targetArch);
         m_deviceInfo = info; // Keep around for later steps
     }
 
@@ -176,25 +139,35 @@ bool AndroidDeployQtStep::init(QList<const BuildStep *> &earlierSteps)
     m_avdName = info.avdname;
     m_serialNumber = info.serialNumber;
 
-    m_appProcessBinaries.clear();
-    m_libdir = QLatin1String("lib");
+    ProjectExplorer::BuildConfiguration *bc = target()->activeBuildConfiguration();
+    m_filesToPull.clear();
+    QString buildDir = bc->buildDirectory().toString();
+    if (!buildDir.endsWith("/")) {
+        buildDir += "/";
+    }
+    QString linkerName("linker");
+    QString libDirName("lib");
     if (info.cpuAbi.contains(QLatin1String("arm64-v8a")) ||
             info.cpuAbi.contains(QLatin1String("x86_64"))) {
         ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit(), ProjectExplorer::Constants::CXX_LANGUAGE_ID);
         if (tc && tc->targetAbi().wordWidth() == 64) {
-            m_appProcessBinaries << QLatin1String("/system/bin/app_process64");
-            m_libdir += QLatin1String("64");
+            m_filesToPull["/system/bin/app_process64"] = buildDir + "app_process";
+            libDirName = "lib64";
+            linkerName = "linker64";
         } else {
-            m_appProcessBinaries << QLatin1String("/system/bin/app_process32");
+            m_filesToPull["/system/bin/app_process32"] = buildDir + "app_process";
         }
     } else {
-        m_appProcessBinaries << QLatin1String("/system/bin/app_process32")
-                             << QLatin1String("/system/bin/app_process");
+        m_filesToPull["/system/bin/app_process32"] = buildDir + "app_process";
+        m_filesToPull["/system/bin/app_process"] = buildDir + "app_process";
     }
 
+    m_filesToPull["/system/bin/" + linkerName] = buildDir + linkerName;
+    m_filesToPull["/system/" + libDirName + "/libc.so"] = buildDir + "libc.so";
+
     AndroidManager::setDeviceSerialNumber(target(), m_serialNumber);
+    AndroidManager::setDeviceApiLevel(target(), info.sdk);
 
-    ProjectExplorer::BuildConfiguration *bc = target()->activeBuildConfiguration();
 
     QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(target()->kit());
     if (!version)
@@ -226,21 +199,12 @@ bool AndroidDeployQtStep::init(QList<const BuildStep *> &earlierSteps)
             return false;
         }
         Utils::QtcProcess::addArg(&m_androiddeployqtArgs, tmp.toString());
-
-        Utils::QtcProcess::addArg(&m_androiddeployqtArgs, QLatin1String("--deployment"));
-        switch (androidBuildApkStep->deployAction()) {
-            case AndroidBuildApkStep::MinistroDeployment:
-                Utils::QtcProcess::addArg(&m_androiddeployqtArgs, QLatin1String("ministro"));
-                break;
-            case AndroidBuildApkStep::DebugDeployment:
-                Utils::QtcProcess::addArg(&m_androiddeployqtArgs, QLatin1String("debug"));
-                break;
-            case AndroidBuildApkStep::BundleLibrariesDeployment:
-                Utils::QtcProcess::addArg(&m_androiddeployqtArgs, QLatin1String("bundled"));
-                break;
+        if (androidBuildApkStep->useMinistro()) {
+            Utils::QtcProcess::addArg(&m_androiddeployqtArgs, QLatin1String("--deployment"));
+            Utils::QtcProcess::addArg(&m_androiddeployqtArgs, QLatin1String("ministro"));
         }
-        if (androidBuildApkStep->useGradle())
-            Utils::QtcProcess::addArg(&m_androiddeployqtArgs, QLatin1String("--gradle"));
+
+        Utils::QtcProcess::addArg(&m_androiddeployqtArgs, QLatin1String("--gradle"));
 
         if (androidBuildApkStep->signPackage()) {
             // The androiddeployqt tool is not really written to do stand-alone installations.
@@ -259,8 +223,6 @@ bool AndroidDeployQtStep::init(QList<const BuildStep *> &earlierSteps)
     }
     m_environment = bc->environment();
 
-    m_buildDirectory = bc->buildDirectory().toString();
-
     m_adbPath = AndroidConfigurations::currentConfig().adbToolPath().toString();
 
     AndroidAvdManager avdManager;
@@ -363,11 +325,12 @@ AndroidDeployQtStep::DeployErrorCode AndroidDeployQtStep::runDeploy(QFutureInter
         emit addOutput(tr("The process \"%1\" crashed.").arg(m_command), BuildStep::OutputFormat::ErrorMessage);
     }
 
-    if (exitCode == 0 && exitStatus == QProcess::NormalExit) {
-        if (deployError != NoError && m_uninstallPreviousPackageRun) {
-            deployError = Failure;
-        }
-    } else {
+    if (deployError != NoError) {
+        if (m_uninstallPreviousPackageRun)
+            deployError = Failure; // Even re-install failed. Set to Failure.
+    } else if (exitCode != 0 || exitStatus != QProcess::NormalExit) {
+        // Set the deployError to Failure when no deployError code was detected
+        // but the adb tool failed otherwise relay the detected deployError.
         deployError = Failure;
     }
 
@@ -405,7 +368,7 @@ void AndroidDeployQtStep::slotAskForUninstall(DeployErrorCode errorCode)
     uninstallMsg.append(tr("\nUninstalling the installed package may solve the issue.\nDo you want to uninstall the existing package?"));
     int button = QMessageBox::critical(0, tr("Install failed"), uninstallMsg,
                                        QMessageBox::Yes, QMessageBox::No);
-    m_askForUinstall = button == QMessageBox::Yes;
+    m_askForUninstall = button == QMessageBox::Yes;
 }
 
 void AndroidDeployQtStep::slotSetSerialNumber(const QString &serialNumber)
@@ -428,38 +391,25 @@ void AndroidDeployQtStep::run(QFutureInterface<bool> &fi)
     DeployErrorCode returnValue = runDeploy(fi);
     if (returnValue > DeployErrorCode::NoError && returnValue < DeployErrorCode::Failure) {
         emit askForUninstall(returnValue);
-        if (m_askForUinstall) {
+        if (m_askForUninstall) {
             m_uninstallPreviousPackageRun = true;
             returnValue = runDeploy(fi);
         }
     }
 
     emit addOutput(tr("Pulling files necessary for debugging."), OutputFormat::NormalMessage);
-
-    QString localAppProcessFile = QString::fromLatin1("%1/app_process").arg(m_buildDirectory);
-    QFile::remove(localAppProcessFile);
-
-    foreach (const QString &remoteAppProcessFile, m_appProcessBinaries) {
+    for (auto itr = m_filesToPull.constBegin(); itr != m_filesToPull.constEnd(); ++itr) {
+        QFile::remove(itr.value());
         runCommand(m_adbPath,
                    AndroidDeviceInfo::adbSelector(m_serialNumber)
-                   << QLatin1String("pull") << remoteAppProcessFile
-                   << localAppProcessFile);
-        if (QFileInfo::exists(localAppProcessFile))
-            break;
+                   << "pull" << itr.key() << itr.value());
+        if (!QFileInfo::exists(itr.value())) {
+            emit addOutput(tr("Package deploy: Failed to pull \"%1\" to \"%2\".")
+                           .arg(itr.key())
+                           .arg(itr.value()), OutputFormat::ErrorMessage);
+        }
     }
 
-    if (!QFileInfo::exists(localAppProcessFile)) {
-        returnValue = Failure;
-        emit addOutput(tr("Package deploy: Failed to pull \"%1\" to \"%2\".")
-                       .arg(m_appProcessBinaries.join(QLatin1String("\", \"")))
-                       .arg(localAppProcessFile), OutputFormat::ErrorMessage);
-    }
-
-    runCommand(m_adbPath,
-               AndroidDeviceInfo::adbSelector(m_serialNumber) << QLatin1String("pull")
-               << QLatin1String("/system/") + m_libdir + QLatin1String("/libc.so")
-               << QString::fromLatin1("%1/libc.so").arg(m_buildDirectory));
-
     reportRunResult(fi, returnValue == NoError);
 }
 
