@@ -150,7 +150,8 @@ QString AndroidToolChain::typeDisplayName() const
 bool AndroidToolChain::isValid() const
 {
     return GccToolChain::isValid() && targetAbi().isValid() && !m_ndkToolChainVersion.isEmpty()
-            && compilerCommand().isChildOf(AndroidConfigurations::currentConfig().ndkLocation());
+            && compilerCommand().isChildOf(AndroidConfigurations::currentConfig().ndkLocation())
+            && !originalTargetTriple().isEmpty();
 }
 
 void AndroidToolChain::addToEnvironment(Environment &env) const
@@ -417,6 +418,9 @@ bool AndroidToolChainFactory::versionCompareLess(const QList<int> &a, const QLis
 bool AndroidToolChainFactory::versionCompareLess(QList<AndroidToolChain *> atc,
                                                  QList<AndroidToolChain *> btc)
 {
+    if (atc.isEmpty() || btc.isEmpty())
+        return false;
+
     const QList<int> a = versionNumberFromString(atc.at(0)->ndkToolChainVersion());
     const QList<int> b = versionNumberFromString(btc.at(0)->ndkToolChainVersion());
 
@@ -463,7 +467,7 @@ AndroidToolChainFactory::autodetectToolChainsForNdk(const FileName &ndkPath,
             FileName compilerPath = AndroidConfigurations::currentConfig().gccPath(abi, lang, version);
 
             AndroidToolChain *tc = findToolChain(compilerPath, lang, alreadyKnown);
-            if (!tc) {
+            if (!tc || tc->originalTargetTriple().isEmpty()) {
                 tc = new AndroidToolChain(abi, version, lang,
                                           ToolChain::AutoDetection);
                 tc->resetToolChain(compilerPath);
@@ -473,6 +477,8 @@ AndroidToolChainFactory::autodetectToolChainsForNdk(const FileName &ndkPath,
             toolChainBundle.append(tc);
         }
 
+        QTC_ASSERT(!toolChainBundle.isEmpty(), continue);
+
         auto it = newestToolChainForArch.constFind(abi);
         if (it == newestToolChainForArch.constEnd())
             newestToolChainForArch.insert(abi, toolChainBundle);
