@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,48 +9,44 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef ICORE_H
-#define ICORE_H
+#pragma once
 
 #include "core_global.h"
 #include "id.h"
 
+#include <QMainWindow>
 #include <QObject>
+#include <QRect>
 #include <QSettings>
 
+#include <functional>
+
 QT_BEGIN_NAMESPACE
 class QPrinter;
 class QStatusBar;
 class QWidget;
-template <class T> class QList;
+template <typename T> class QList;
 QT_END_NAMESPACE
 
 namespace Core {
 class IWizardFactory;
 class Context;
 class IContext;
-class ProgressManager;
 class SettingsDatabase;
-class VcsManager;
 
 namespace Internal { class MainWindow; }
 
@@ -59,16 +55,24 @@ class CORE_EXPORT ICore : public QObject
     Q_OBJECT
 
     friend class Internal::MainWindow;
+    friend class IWizardFactory;
+
     explicit ICore(Internal::MainWindow *mw);
     ~ICore();
 
 public:
+    enum class ContextPriority {
+        High,
+        Low
+    };
+
     // This should only be used to acccess the signals, so it could
     // theoretically return an QObject *. For source compatibility
     // it returns a ICore.
     static ICore *instance();
 
     static bool isNewItemDialogRunning();
+    static QWidget *newItemDialog();
     static void showNewItemDialog(const QString &title,
                                   const QList<IWizardFactory *> &factories,
                                   const QString &defaultLocation = QString(),
@@ -90,23 +94,27 @@ public:
 
     static QString resourcePath();
     static QString userResourcePath();
+    static QString installerResourcePath();
     static QString documentationPath();
     static QString libexecPath();
 
     static QString versionString();
     static QString buildCompatibilityString();
 
-    static QWidget *mainWindow();
+    static QMainWindow *mainWindow();
     static QWidget *dialogParent();
     static QStatusBar *statusBar();
     /* Raises and activates the window for the widget. This contains workarounds for X11. */
     static void raiseWindow(QWidget *widget);
 
     static IContext *currentContextObject();
+    static QWidget *currentContextWidget();
     // Adds and removes additional active contexts, these contexts are appended
     // to the currently active contexts.
-    static void updateAdditionalContexts(const Context &remove, const Context &add);
-    static void addAdditionalContext(const Context &context);
+    static void updateAdditionalContexts(const Context &remove, const Context &add,
+                                         ContextPriority priority = ContextPriority::Low);
+    static void addAdditionalContext(const Context &context,
+                                     ContextPriority priority = ContextPriority::Low);
     static void removeAdditionalContext(const Context &context);
     static void addContextObject(IContext *context);
     static void removeContextObject(IContext *context);
@@ -117,13 +125,16 @@ public:
     enum OpenFilesFlags {
         None = 0,
         SwitchMode = 1,
-        CanContainLineNumbers = 2,
+        CanContainLineAndColumnNumbers = 2,
          /// Stop loading once the first file fails to load
         StopOnLoadFail = 4
     };
     static void openFiles(const QStringList &fileNames, OpenFilesFlags flags = None);
 
-    static void emitNewItemsDialogRequested();
+    static void addPreCloseListener(const std::function<bool()> &listener);
+
+    static QString systemInformation();
+    static void setupScreenShooter(const QString &name, QWidget *w, const QRect &rc = QRect());
 
 public slots:
     static void saveSettings();
@@ -131,16 +142,20 @@ public slots:
 signals:
     void coreAboutToOpen();
     void coreOpened();
-    void newItemsDialogRequested();
-    void newItemDialogRunningChanged();
+    void newItemDialogStateChanged();
     void saveSettingsRequested();
     void optionsDialogRequested();
     void coreAboutToClose();
     void contextAboutToChange(const QList<Core::IContext *> &context);
-    void contextChanged(const QList<Core::IContext *> &context, const Core::Context &additionalContexts);
-    void themeChanged();
+    void contextChanged(const Core::Context &context);
+
+public:
+    /* internal use */
+    static QStringList additionalAboutInformation();
+    static void appendAboutInformation(const QString &line);
+
+private:
+    static void updateNewItemDialogState();
 };
 
 } // namespace Core
-
-#endif // ICORE_H
