@@ -25,7 +25,7 @@
 
 #include "cppcodemodelinspectordialog.h"
 #include "ui_cppcodemodelinspectordialog.h"
-#include "cppeditor.h"
+#include "cppeditorwidget.h"
 #include "cppeditordocument.h"
 
 #include <coreplugin/editormanager/editormanager.h>
@@ -35,6 +35,7 @@
 #include <cpptools/cppmodelmanager.h>
 #include <cpptools/cpptoolsbridge.h>
 #include <cpptools/cppworkingcopy.h>
+#include <projectexplorer/projectmacro.h>
 #include <projectexplorer/project.h>
 
 #include <cplusplus/CppDocument.h>
@@ -48,6 +49,9 @@
 #include <QPushButton>
 #include <QSortFilterProxyModel>
 
+#include <algorithm>
+#include <numeric>
+
 using namespace CPlusPlus;
 using namespace CppTools;
 namespace CMI = CppCodeModelInspector;
@@ -199,6 +203,162 @@ void FilterableView::clearFilter()
     lineEdit->clear();
 }
 
+// --- ProjectFilesModel --------------------------------------------------------------------------
+
+class ProjectFilesModel : public QAbstractListModel
+{
+    Q_OBJECT
+public:
+    ProjectFilesModel(QObject *parent);
+    void configure(const ProjectFiles &files);
+    void clear();
+
+    enum Columns { FileKindColumn, FilePathColumn, ColumnCount };
+
+    int rowCount(const QModelIndex &parent = QModelIndex()) const;
+    int columnCount(const QModelIndex &parent = QModelIndex()) const;
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
+    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
+
+private:
+    ProjectFiles m_files;
+};
+
+ProjectFilesModel::ProjectFilesModel(QObject *parent) : QAbstractListModel(parent)
+{
+}
+
+void ProjectFilesModel::configure(const ProjectFiles &files)
+{
+    emit layoutAboutToBeChanged();
+    m_files = files;
+    emit layoutChanged();
+}
+
+void ProjectFilesModel::clear()
+{
+    emit layoutAboutToBeChanged();
+    m_files.clear();
+    emit layoutChanged();
+}
+
+int ProjectFilesModel::rowCount(const QModelIndex &/*parent*/) const
+{
+    return m_files.size();
+}
+
+int ProjectFilesModel::columnCount(const QModelIndex &/*parent*/) const
+{
+    return ProjectFilesModel::ColumnCount;
+}
+
+QVariant ProjectFilesModel::data(const QModelIndex &index, int role) const
+{
+    if (role == Qt::DisplayRole) {
+        const int row = index.row();
+        const int column = index.column();
+        if (column == FileKindColumn) {
+            return CMI::Utils::toString(m_files.at(row).kind);
+        } else if (column == FilePathColumn) {
+            return m_files.at(row).path;
+        }
+    }
+    return QVariant();
+}
+
+QVariant ProjectFilesModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+    if (orientation == Qt::Horizontal && role == Qt::DisplayRole) {
+        switch (section) {
+        case FileKindColumn:
+            return QLatin1String("File Kind");
+        case FilePathColumn:
+            return QLatin1String("File Path");
+        default:
+            return QVariant();
+        }
+    }
+    return QVariant();
+}
+
+// --- ProjectHeaderPathModel --------------------------------------------------------------------
+
+class ProjectHeaderPathsModel : public QAbstractListModel
+{
+    Q_OBJECT
+public:
+    ProjectHeaderPathsModel(QObject *parent);
+    void configure(const ProjectPartHeaderPaths &paths);
+    void clear();
+
+    enum Columns { TypeColumn, PathColumn, ColumnCount };
+
+    int rowCount(const QModelIndex &parent = QModelIndex()) const;
+    int columnCount(const QModelIndex &parent = QModelIndex()) const;
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
+    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
+
+private:
+    ProjectPartHeaderPaths m_paths;
+};
+
+ProjectHeaderPathsModel::ProjectHeaderPathsModel(QObject *parent) : QAbstractListModel(parent)
+{
+}
+
+void ProjectHeaderPathsModel::configure(const ProjectPartHeaderPaths &paths)
+{
+    emit layoutAboutToBeChanged();
+    m_paths = paths;
+    emit layoutChanged();
+}
+
+void ProjectHeaderPathsModel::clear()
+{
+    emit layoutAboutToBeChanged();
+    m_paths.clear();
+    emit layoutChanged();
+}
+
+int ProjectHeaderPathsModel::rowCount(const QModelIndex &/*parent*/) const
+{
+    return m_paths.size();
+}
+
+int ProjectHeaderPathsModel::columnCount(const QModelIndex &/*parent*/) const
+{
+    return ProjectFilesModel::ColumnCount;
+}
+
+QVariant ProjectHeaderPathsModel::data(const QModelIndex &index, int role) const
+{
+    if (role == Qt::DisplayRole) {
+        const int row = index.row();
+        const int column = index.column();
+        if (column == TypeColumn) {
+            return CMI::Utils::toString(m_paths.at(row).type);
+        } else if (column == PathColumn) {
+            return m_paths.at(row).path;
+        }
+    }
+    return QVariant();
+}
+
+QVariant ProjectHeaderPathsModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+    if (orientation == Qt::Horizontal && role == Qt::DisplayRole) {
+        switch (section) {
+        case TypeColumn:
+            return QLatin1String("Type");
+        case PathColumn:
+            return QLatin1String("Path");
+        default:
+            return QVariant();
+        }
+    }
+    return QVariant();
+}
+
 // --- KeyValueModel ------------------------------------------------------------------------------
 
 class KeyValueModel : public QAbstractListModel
@@ -410,7 +570,7 @@ void IncludesModel::configure(const QList<Document::Include> &includes)
 {
     emit layoutAboutToBeChanged();
     m_includes = includes;
-    qStableSort(m_includes.begin(), m_includes.end(), includesSorter);
+    std::stable_sort(m_includes.begin(), m_includes.end(), includesSorter);
     emit layoutChanged();
 }
 
@@ -513,7 +673,7 @@ void DiagnosticMessagesModel::configure(
 {
     emit layoutAboutToBeChanged();
     m_messages = messages;
-    qStableSort(m_messages.begin(), m_messages.end(), diagnosticMessagesModelSorter);
+    std::stable_sort(m_messages.begin(), m_messages.end(), diagnosticMessagesModelSorter);
     emit layoutChanged();
 }
 
@@ -598,7 +758,7 @@ class MacrosModel : public QAbstractListModel
     Q_OBJECT
 public:
     MacrosModel(QObject *parent);
-    void configure(const QList<Macro> &macros);
+    void configure(const QList<CPlusPlus::Macro> &macros);
     void clear();
 
     enum Columns { LineNumberColumn, MacroColumn, ColumnCount };
@@ -609,14 +769,14 @@ public:
     QVariant headerData(int section, Qt::Orientation orientation, int role) const;
 
 private:
-    QList<Macro> m_macros;
+    QList<CPlusPlus::Macro> m_macros;
 };
 
 MacrosModel::MacrosModel(QObject *parent) : QAbstractListModel(parent)
 {
 }
 
-void MacrosModel::configure(const QList<Macro> &macros)
+void MacrosModel::configure(const QList<CPlusPlus::Macro> &macros)
 {
     emit layoutAboutToBeChanged();
     m_macros = macros;
@@ -644,7 +804,7 @@ QVariant MacrosModel::data(const QModelIndex &index, int role) const
 {
     const int column = index.column();
     if (role == Qt::DisplayRole || (role == Qt::ToolTipRole && column == MacroColumn)) {
-        const Macro macro = m_macros.at(index.row());
+        const CPlusPlus::Macro macro = m_macros.at(index.row());
         if (column == LineNumberColumn)
             return macro.line();
         else if (column == MacroColumn)
@@ -1021,6 +1181,11 @@ QVariant ProjectPartsModel::data(const QModelIndex &index, int role) const
             return m_projectPartsList.at(row)->displayName;
         else if (column == PartFilePathColumn)
             return QDir::toNativeSeparators(m_projectPartsList.at(row)->projectFile);
+    } else if (role == Qt::ForegroundRole) {
+        if (!m_projectPartsList.at(row)->selectedForBuilding) {
+            return QApplication::palette().color(QPalette::ColorGroup::Disabled,
+                                                 QPalette::ColorRole::Text);
+        }
     } else if (role == Qt::UserRole) {
         return m_projectPartsList.at(row)->id();
     }
@@ -1173,6 +1338,8 @@ CppCodeModelInspectorDialog::CppCodeModelInspectorDialog(QWidget *parent)
     , m_projectPartsModel(new ProjectPartsModel(this))
     , m_proxyProjectPartsModel(new QSortFilterProxyModel(this))
     , m_partGenericInfoModel(new KeyValueModel(this))
+    , m_projectFilesModel(new ProjectFilesModel(this))
+    , m_projectHeaderPathsModel(new ProjectHeaderPathsModel(this))
     , m_workingCopyView(new FilterableView(this))
     , m_workingCopyModel(new WorkingCopyModel(this))
     , m_proxyWorkingCopyModel(new QSortFilterProxyModel(this))
@@ -1199,6 +1366,8 @@ CppCodeModelInspectorDialog::CppCodeModelInspectorDialog(QWidget *parent)
     m_proxyProjectPartsModel->setFilterKeyColumn(ProjectPartsModel::PartFilePathColumn);
     m_projectPartsView->setModel(m_proxyProjectPartsModel);
     m_ui->partGeneralView->setModel(m_partGenericInfoModel);
+    m_ui->projectFilesView->setModel(m_projectFilesModel);
+    m_ui->projectHeaderPathsView->setModel(m_projectHeaderPathsModel);
 
     m_proxyWorkingCopyModel->setSourceModel(m_workingCopyModel);
     m_proxyWorkingCopyModel->setFilterKeyColumn(WorkingCopyModel::FilePathColumn);
@@ -1413,7 +1582,7 @@ void CppCodeModelInspectorDialog::refresh()
 
     // Project Parts
     const ProjectPart::Ptr editorsProjectPart = cppEditorDocument
-        ? cppEditorDocument->processor()->parser()->projectPart()
+        ? cppEditorDocument->processor()->parser()->projectPartInfo().projectPart
         : ProjectPart::Ptr();
 
     const QList<ProjectInfo> projectInfos = cmmi->projectInfos();
@@ -1447,7 +1616,8 @@ void CppCodeModelInspectorDialog::refresh()
     }
 
     // Merged entities
-    dumper.dumpMergedEntities(cmmi->headerPaths(), cmmi->definedMacros());
+    dumper.dumpMergedEntities(cmmi->headerPaths(),
+                              ProjectExplorer::Macro::toByteArray(cmmi->definedMacros()));
 }
 
 enum DocumentTabs {
@@ -1503,24 +1673,16 @@ void CppCodeModelInspectorDialog::updateDocumentData(const Document::Ptr &docume
     QTC_ASSERT(document, return);
 
     // General
-    KeyValueModel::Table table = KeyValueModel::Table()
-        << qMakePair(QString::fromLatin1("File Path"),
-                     QDir::toNativeSeparators(document->fileName()))
-        << qMakePair(QString::fromLatin1("Last Modified"),
-                     CMI::Utils::toString(document->lastModified()))
-        << qMakePair(QString::fromLatin1("Revision"),
-                     CMI::Utils::toString(document->revision()))
-        << qMakePair(QString::fromLatin1("Editor Revision"),
-                     CMI::Utils::toString(document->editorRevision()))
-        << qMakePair(QString::fromLatin1("Check Mode"),
-                     CMI::Utils::toString(document->checkMode()))
-        << qMakePair(QString::fromLatin1("Tokenized"),
-                     CMI::Utils::toString(document->isTokenized()))
-        << qMakePair(QString::fromLatin1("Parsed"),
-                     CMI::Utils::toString(document->isParsed()))
-        << qMakePair(QString::fromLatin1("Project Parts"),
-                     CMI::Utils::partsForFile(document->fileName()))
-        ;
+    const KeyValueModel::Table table = {
+        {QString::fromLatin1("File Path"), QDir::toNativeSeparators(document->fileName())},
+        {QString::fromLatin1("Last Modified"), CMI::Utils::toString(document->lastModified())},
+        {QString::fromLatin1("Revision"), CMI::Utils::toString(document->revision())},
+        {QString::fromLatin1("Editor Revision"), CMI::Utils::toString(document->editorRevision())},
+        {QString::fromLatin1("Check Mode"), CMI::Utils::toString(document->checkMode())},
+        {QString::fromLatin1("Tokenized"), CMI::Utils::toString(document->isTokenized())},
+        {QString::fromLatin1("Parsed"), CMI::Utils::toString(document->isParsed())},
+        {QString::fromLatin1("Project Parts"), CMI::Utils::partsForFile(document->fileName())}
+    };
     m_docGenericInfoModel->configure(table);
     resizeColumns<KeyValueModel>(m_ui->docGeneralView);
 
@@ -1582,15 +1744,15 @@ static QString partTabName(int tabIndex, int numberOfEntries = -1)
 void CppCodeModelInspectorDialog::clearProjectPartData()
 {
     m_partGenericInfoModel->clear();
+    m_projectFilesModel->clear();
+    m_projectHeaderPathsModel->clear();
 
-    m_ui->partProjectFilesEdit->clear();
     m_ui->projectPartTab->setTabText(ProjectPartFilesTab, partTabName(ProjectPartFilesTab));
 
     m_ui->partToolchainDefinesEdit->clear();
     m_ui->partProjectDefinesEdit->clear();
     m_ui->projectPartTab->setTabText(ProjectPartDefinesTab, partTabName(ProjectPartDefinesTab));
 
-    m_ui->partHeaderPathsEdit->clear();
     m_ui->projectPartTab->setTabText(ProjectPartHeaderPathsTab,
                                      partTabName(ProjectPartHeaderPathsTab));
 
@@ -1599,6 +1761,15 @@ void CppCodeModelInspectorDialog::clearProjectPartData()
                                      partTabName(ProjectPartPrecompiledHeadersTab));
 }
 
+static int defineCount(const ProjectExplorer::Macros &macros)
+{
+    using ProjectExplorer::Macro;
+    return int(std::count_if(
+                   macros.begin(),
+                   macros.end(),
+                   [](const Macro &macro) { return macro.type == ProjectExplorer::MacroType::Define; }));
+}
+
 void CppCodeModelInspectorDialog::updateProjectPartData(const ProjectPart::Ptr &part)
 {
     QTC_ASSERT(part, return);
@@ -1610,45 +1781,49 @@ void CppCodeModelInspectorDialog::updateProjectPartData(const ProjectPart::Ptr &
         projectName = project->displayName();
         projectFilePath = project->projectFilePath().toUserOutput();
     }
-    KeyValueModel::Table table = KeyValueModel::Table()
-        << qMakePair(QString::fromLatin1("Project Part Name"), part->displayName)
-        << qMakePair(QString::fromLatin1("Project Part File"),
-                     QDir::toNativeSeparators(part->projectFile))
-        << qMakePair(QString::fromLatin1("Project Name"), projectName)
-        << qMakePair(QString::fromLatin1("Project File"), projectFilePath)
-        << qMakePair(QString::fromLatin1("Language Version"),
-                     CMI::Utils::toString(part->languageVersion))
-        << qMakePair(QString::fromLatin1("Language Extensions"),
-                     CMI::Utils::toString(part->languageExtensions))
-        << qMakePair(QString::fromLatin1("Qt Version"),
-                     CMI::Utils::toString(part->qtVersion))
-        ;
+    const QString callGroupId = part->callGroupId.isEmpty() ? QString::fromLatin1("<None>")
+                                                            : part->callGroupId;
+    const QString buildSystemTarget
+            = part->buildSystemTarget.isEmpty() ? QString::fromLatin1("<None>")
+                                                : part->buildSystemTarget;
+
+    const QString precompiledHeaders = part->precompiledHeaders.isEmpty()
+            ? QString::fromLatin1("<None>")
+            : part->precompiledHeaders.join(',');
+
+    KeyValueModel::Table table = {
+        {QString::fromLatin1("Project Part Name"), part->displayName},
+        {QString::fromLatin1("Project Part File"), part->projectFileLocation()},
+        {QString::fromLatin1("Project Name"), projectName},
+        {QString::fromLatin1("Project File"), projectFilePath},
+        {QString::fromLatin1("Buildsystem Target"), buildSystemTarget},
+        {QString::fromLatin1("Callgroup Id"), callGroupId},
+        {QString::fromLatin1("Precompiled Headers"), precompiledHeaders},
+        {QString::fromLatin1("Selected For Building"), CMI::Utils::toString(part->selectedForBuilding)},
+        {QString::fromLatin1("Build Target Type"), CMI::Utils::toString(part->buildTargetType)},
+        {QString::fromLatin1("Language Version"), CMI::Utils::toString(part->languageVersion)},
+        {QString::fromLatin1("Language Extensions"), CMI::Utils::toString(part->languageExtensions)},
+        {QString::fromLatin1("Qt Version"), CMI::Utils::toString(part->qtVersion)}
+    };
     if (!part->projectConfigFile.isEmpty())
-        table.prepend(qMakePair(QString::fromLatin1("Project Config File"),
-                                part->projectConfigFile));
+        table.prepend({QString::fromLatin1("Project Config File"), part->projectConfigFile});
     m_partGenericInfoModel->configure(table);
     resizeColumns<KeyValueModel>(m_ui->partGeneralView);
 
     // Project Files
-    m_ui->partProjectFilesEdit->setPlainText(CMI::Utils::toString(part->files));
+    m_projectFilesModel->configure(part->files);
     m_ui->projectPartTab->setTabText(ProjectPartFilesTab,
         partTabName(ProjectPartFilesTab, part->files.size()));
 
-    // Defines
-    const QList<QByteArray> defineLines = part->toolchainDefines.split('\n')
-        + part->projectDefines.split('\n');
-    int numberOfDefines = 0;
-    foreach (const QByteArray &line, defineLines) {
-        if (line.startsWith("#define "))
-            ++numberOfDefines;
-    }
-    m_ui->partToolchainDefinesEdit->setPlainText(QString::fromUtf8(part->toolchainDefines));
-    m_ui->partProjectDefinesEdit->setPlainText(QString::fromUtf8(part->projectDefines));
+    int numberOfDefines = defineCount(part->toolChainMacros) + defineCount(part->projectMacros);
+
+    m_ui->partToolchainDefinesEdit->setPlainText(QString::fromUtf8(ProjectExplorer::Macro::toByteArray(part->toolChainMacros)));
+    m_ui->partProjectDefinesEdit->setPlainText(QString::fromUtf8(ProjectExplorer::Macro::toByteArray(part->projectMacros)));
     m_ui->projectPartTab->setTabText(ProjectPartDefinesTab,
         partTabName(ProjectPartDefinesTab, numberOfDefines));
 
     // Header Paths
-    m_ui->partHeaderPathsEdit->setPlainText(CMI::Utils::pathListToString(part->headerPaths));
+    m_projectHeaderPathsModel->configure(part->headerPaths);
     m_ui->projectPartTab->setTabText(ProjectPartHeaderPathsTab,
         partTabName(ProjectPartHeaderPathsTab, part->headerPaths.size()));
 
