@@ -33,11 +33,6 @@
 
 namespace CppTools {
 
-static QStringList commonWarnings()
-{
-    return { QStringLiteral("-Wno-unknown-pragmas") };
-}
-
 static void addConfigForQuestionableConstructs(ClangDiagnosticConfigsModel &model)
 {
     ClangDiagnosticConfig config;
@@ -48,7 +43,7 @@ static void addConfigForQuestionableConstructs(ClangDiagnosticConfigsModel &mode
     config.setClangOptions(QStringList{
         QStringLiteral("-Wall"),
         QStringLiteral("-Wextra"),
-    } + commonWarnings());
+    });
 
     model.appendOrUpdate(config);
 }
@@ -60,7 +55,7 @@ static void addConfigForPedanticWarnings(ClangDiagnosticConfigsModel &model)
     config.setDisplayName(QCoreApplication::translate("ClangDiagnosticConfigsModel",
                                                       "Pedantic Warnings"));
     config.setIsReadOnly(true);
-    config.setClangOptions(QStringList{QStringLiteral("-Wpedantic")} + commonWarnings());
+    config.setClangOptions(QStringList{QStringLiteral("-Wpedantic")});
 
     model.appendOrUpdate(config);
 }
@@ -86,7 +81,7 @@ static void addConfigForAlmostEveryWarning(ClangDiagnosticConfigsModel &model)
         QStringLiteral("-Wno-switch-enum"),
         QStringLiteral("-Wno-missing-prototypes"), // Not optimal for C projects.
         QStringLiteral("-Wno-used-but-marked-unused"), // e.g. QTest::qWait
-    } + commonWarnings());
+    });
 
     model.appendOrUpdate(config);
 }
