@@ -30,23 +30,14 @@
 
 #include <projectexplorer/devicesupport/idevice.h>
 
-#include <QSharedPointer>
-#include <QProcess>
-#include <QMap>
-#include <QVariant>
 #include <QTime>
 
-#include <functional>
-
-namespace Utils { class ConsoleProcess; }
 namespace Debugger {
 namespace Internal {
 
-class DisassemblerAgent;
 class CdbCommand;
 struct MemoryViewCookie;
 class StringInputStream;
-class GdbMi;
 
 class CdbEngine : public DebuggerEngine
 {
@@ -56,7 +47,7 @@ public:
     typedef QSharedPointer<CdbCommand> CdbCommandPtr;
     typedef std::function<void(const DebuggerResponse &)> CommandHandler;
 
-    CdbEngine(const DebuggerRunParameters &sp);
+    explicit CdbEngine();
     ~CdbEngine() override;
 
     // Factory function that returns 0 if the debug engine library cannot be found.
@@ -66,11 +57,10 @@ public:
     DebuggerEngine *cppEngine() override { return this; }
 
     void setupEngine() override;
-    void setupInferior() override;
     void runEngine() override;
     void shutdownInferior() override;
     void shutdownEngine() override;
-    void abortDebugger() override;
+    void abortDebuggerProcess() override;
     void detachDebugger() override;
     bool hasCapability(unsigned cap) const override;
     void watchPoint(const QPoint &) override;
@@ -99,9 +89,8 @@ public:
     void attemptBreakpointSynchronization() override;
 
     void fetchDisassembler(DisassemblerAgent *agent) override;
-    void fetchMemory(MemoryAgent *, QObject *, quint64 addr, quint64 length) override;
-    void changeMemory(Internal::MemoryAgent *, QObject *, quint64 addr,
-                      const QByteArray &data) override;
+    void fetchMemory(MemoryAgent *, quint64 addr, quint64 length) override;
+    void changeMemory(MemoryAgent *, quint64 addr, const QByteArray &data) override;
 
     void reloadModules() override;
     void loadSymbols(const QString &moduleName) override;
@@ -124,10 +113,6 @@ private:
     void runCommand(const DebuggerCommand &cmd) override;
     void operateByInstructionTriggered(bool);
 
-    void consoleStubError(const QString &);
-    void consoleStubProcessStarted();
-    void consoleStubExited();
-
     void createFullBacktrace();
 
     void handleDoInterruptInferior(const QString &errorMessage);
@@ -159,9 +144,9 @@ private:
         NoFlags = 0,
         BuiltinCommand,
         ExtensionCommand,
+        ScriptCommand
     };
 
-    bool startConsole(const DebuggerRunParameters &sp, QString *errorMessage);
     void init();
     unsigned examineStopReason(const GdbMi &stopReason, QString *message,
                                QString *exceptionBoxMessage,
@@ -170,7 +155,6 @@ private:
     bool commandsPending() const;
     void handleExtensionMessage(char t, int token, const QString &what, const QString &message);
     bool doSetupEngine(QString *errorMessage);
-    bool launchCDB(const DebuggerRunParameters &sp, QString *errorMessage);
     void handleSessionAccessible(unsigned long cdbExState);
     void handleSessionInaccessible(unsigned long cdbExState);
     void handleSessionIdle(const QString &message);
@@ -189,6 +173,8 @@ private:
                                  DisassemblerAgent *agent);
     void postResolveSymbol(const QString &module, const QString &function,
                            DisassemblerAgent *agent);
+    void showScriptMessages(const QString &message) const;
+    void handleInitialSessionIdle();
     // Builtin commands
     void handleStackTrace(const DebuggerResponse &);
     void handleRegisters(const DebuggerResponse &);
@@ -211,6 +197,7 @@ private:
     void handleWidgetAt(const DebuggerResponse &response);
     void handleBreakPoints(const DebuggerResponse &response);
     void handleAdditionalQmlStack(const DebuggerResponse &response);
+    void setupScripting(const DebuggerResponse &response);
     NormalizedSourceFileName sourceMapNormalizeFileNameFromDebugger(const QString &f);
     void doUpdateLocals(const UpdateParameters &params) override;
     void updateAll() override;
@@ -219,46 +206,48 @@ private:
     void mergeStartParametersSourcePathMap();
 
     const QString m_tokenPrefix;
+    void handleSetupFailure(const QString &errorMessage);
 
     QProcess m_process;
-    QScopedPointer<Utils::ConsoleProcess> m_consoleStub;
-    DebuggerStartMode m_effectiveStartMode;
+    DebuggerStartMode m_effectiveStartMode = NoStartMode;
     QByteArray m_outputBuffer;
     //! Debugger accessible (expecting commands)
-    bool m_accessible;
-    SpecialStopMode m_specialStopMode;
+    bool m_accessible = false;
+    SpecialStopMode m_specialStopMode = NoSpecialStop;
     ProjectExplorer::DeviceProcessSignalOperation::Ptr m_signalOperation;
-    int m_nextCommandToken;
+    int m_nextCommandToken = 0;
     QHash<int, DebuggerCommand> m_commandForToken;
     QString m_currentBuiltinResponse;
-    int m_currentBuiltinResponseToken;
+    int m_currentBuiltinResponseToken = -1;
     QMap<QString, NormalizedSourceFileName> m_normalizedFileCache;
     const QString m_extensionCommandPrefix; //!< Library name used as prefix
-    bool m_operateByInstructionPending; //!< Creator operate by instruction action changed.
-    bool m_operateByInstruction;
-    bool m_hasDebuggee;
+    bool m_operateByInstructionPending = true; //!< Creator operate by instruction action changed.
+    bool m_operateByInstruction = true; // Default CDB setting.
+    bool m_hasDebuggee = false;
     enum Wow64State {
         wow64Uninitialized,
         noWow64Stack,
         wow64Stack32Bit,
         wow64Stack64Bit
-    } m_wow64State;
+    } m_wow64State = wow64Uninitialized;
     QTime m_logTime;
-    mutable int m_elapsedLogTime;
+    mutable int m_elapsedLogTime = 0;
     QString m_extensionMessageBuffer;
-    bool m_sourceStepInto;
-    int m_watchPointX;
-    int m_watchPointY;
+    bool m_sourceStepInto = false;
+    int m_watchPointX = 0;
+    int m_watchPointY = 0;
     PendingBreakPointMap m_pendingBreakpointMap;
     PendingBreakPointMap m_insertSubBreakpointMap;
     PendingBreakPointMap m_pendingSubBreakpointMap;
-    bool m_autoBreakPointCorrection;
+    bool m_autoBreakPointCorrection = false;
     QHash<QString, QString> m_fileNameModuleHash;
     QMultiHash<QString, quint64> m_symbolAddressCache;
-    bool m_ignoreCdbOutput;
+    bool m_ignoreCdbOutput = false;
     QVariantList m_customSpecialStopData;
     QList<SourcePathMapping> m_sourcePathMappings;
     QScopedPointer<GdbMi> m_coreStopReason;
+    int m_pythonVersion = 0; // 0xMMmmpp MM = major; mm = minor; pp = patch
+    bool m_initialSessionIdleHandled = false;
 };
 
 } // namespace Internal
