@@ -24,14 +24,19 @@
 ****************************************************************************/
 
 #include "unifieddiffeditorwidget.h"
+
+#include "diffeditorconstants.h"
 #include "diffeditordocument.h"
 #include "diffutils.h"
 
+#include <QHash>
 #include <QMenu>
 #include <QPainter>
 #include <QScrollBar>
 #include <QTextBlock>
 
+#include <coreplugin/icore.h>
+
 #include <texteditor/textdocument.h>
 #include <texteditor/textdocumentlayout.h>
 #include <texteditor/texteditorsettings.h>
@@ -53,8 +58,6 @@ UnifiedDiffEditorWidget::UnifiedDiffEditorWidget(QWidget *parent)
     DisplaySettings settings = displaySettings();
     settings.m_textWrapping = false;
     settings.m_displayLineNumbers = true;
-    settings.m_highlightCurrentLine = false;
-    settings.m_displayFoldingMarkers = true;
     settings.m_markTextChanges = false;
     settings.m_highlightBlocks = false;
     SelectableTextEditorWidget::setDisplaySettings(settings);
@@ -73,6 +76,17 @@ UnifiedDiffEditorWidget::UnifiedDiffEditorWidget(QWidget *parent)
 
     connect(this, &QPlainTextEdit::cursorPositionChanged,
             this, &UnifiedDiffEditorWidget::slotCursorPositionChangedInEditor);
+
+    m_context = new Core::IContext(this);
+    m_context->setWidget(this);
+    m_context->setContext(Core::Context(Constants::UNIFIED_VIEW_ID));
+    Core::ICore::addContextObject(m_context);
+    setCodeFoldingSupported(true);
+}
+
+UnifiedDiffEditorWidget::~UnifiedDiffEditorWidget()
+{
+    Core::ICore::removeContextObject(m_context);
 }
 
 void UnifiedDiffEditorWidget::setDocument(DiffEditorDocument *document)
@@ -117,6 +131,9 @@ void UnifiedDiffEditorWidget::setDisplaySettings(const DisplaySettings &ds)
 {
     DisplaySettings settings = displaySettings();
     settings.m_visualizeWhitespace = ds.m_visualizeWhitespace;
+    settings.m_displayFoldingMarkers = ds.m_displayFoldingMarkers;
+    settings.m_scrollBarHighlights = ds.m_scrollBarHighlights;
+    settings.m_highlightCurrentLine = ds.m_highlightCurrentLine;
     SelectableTextEditorWidget::setDisplaySettings(settings);
 }
 
@@ -147,6 +164,16 @@ void UnifiedDiffEditorWidget::mouseDoubleClickEvent(QMouseEvent *e)
     SelectableTextEditorWidget::mouseDoubleClickEvent(e);
 }
 
+void UnifiedDiffEditorWidget::keyPressEvent(QKeyEvent *e)
+{
+    if (e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return) {
+        jumpToOriginalFile(textCursor());
+        e->accept();
+        return;
+    }
+    SelectableTextEditorWidget::keyPressEvent(e);
+}
+
 void UnifiedDiffEditorWidget::contextMenuEvent(QContextMenuEvent *e)
 {
     QPointer<QMenu> menu = createStandardContextMenu();
@@ -163,14 +190,15 @@ void UnifiedDiffEditorWidget::contextMenuEvent(QContextMenuEvent *e)
 }
 
 void UnifiedDiffEditorWidget::addContextMenuActions(QMenu *menu,
-                                                    int diffFileIndex,
+                                                    int fileIndex,
                                                     int chunkIndex)
 {
     menu->addSeparator();
 
-    m_controller.addCodePasterAction(menu);
-    m_controller.addApplyAction(menu, diffFileIndex, chunkIndex);
-    m_controller.addRevertAction(menu, diffFileIndex, chunkIndex);
+    m_controller.addCodePasterAction(menu, fileIndex, chunkIndex);
+    m_controller.addApplyAction(menu, fileIndex, chunkIndex);
+    m_controller.addRevertAction(menu, fileIndex, chunkIndex);
+    m_controller.addExtraActions(menu, fileIndex, chunkIndex);
 }
 
 void UnifiedDiffEditorWidget::clear(const QString &message)
@@ -203,15 +231,15 @@ QString UnifiedDiffEditorWidget::lineNumber(int blockNumber) const
                 ? QString::number(m_leftLineNumbers.value(blockNumber))
                 : QString();
         lineNumberString += QString(m_leftLineNumberDigits - leftLine.count(),
-                                    QLatin1Char(' ')) + leftLine;
+                                    ' ') + leftLine;
 
-        lineNumberString += QLatin1Char('|');
+        lineNumberString += '|';
 
         const QString rightLine = rightLineExists
                 ? QString::number(m_rightLineNumbers.value(blockNumber))
                 : QString();
         lineNumberString += QString(m_rightLineNumberDigits - rightLine.count(),
-                                    QLatin1Char(' ')) + rightLine;
+                                    ' ') + rightLine;
     }
     return lineNumberString;
 }
@@ -301,23 +329,23 @@ QString UnifiedDiffEditorWidget::showChunk(const ChunkData &chunkData,
                 for (int j = 0; j < leftBuffer.count(); j++) {
                     const TextLineData &lineData = leftBuffer.at(j);
                     const QString line = DiffUtils::makePatchLine(
-                                QLatin1Char('-'),
+                                '-',
                                 lineData.text,
                                 lastChunk,
                                 i == chunkData.rows.count()
                                 && j == leftBuffer.count() - 1);
 
-                    const int blockDelta = line.count(QLatin1Char('\n')); // no new line
+                    const int blockDelta = line.count('\n'); // no new line
                                                      // could have been added
                     for (int k = 0; k < blockDelta; k++)
                         (*selections)[*blockNumber + blockCount + 1 + k].append(&m_controller.m_leftLineFormat);
-                    QMapIterator<int, int> itPos(lineData.changedPositions);
-                    while (itPos.hasNext()) {
-                        itPos.next();
-                        const int startPos = itPos.key() < 0
-                                ? 1 : itPos.key() + 1;
-                        const int endPos = itPos.value() < 0
-                                ? itPos.value() : itPos.value() + 1;
+
+                    for (auto it = lineData.changedPositions.cbegin(),
+                              end = lineData.changedPositions.cend(); it != end; ++it) {
+                        const int startPos = it.key() < 0
+                                ? 1 : it.key() + 1;
+                        const int endPos = it.value() < 0
+                                ? it.value() : it.value() + 1;
                         (*selections)[*blockNumber + blockCount + 1].append(
                                     DiffSelection(startPos, endPos, &m_controller.m_leftCharFormat));
                     }
@@ -340,24 +368,24 @@ QString UnifiedDiffEditorWidget::showChunk(const ChunkData &chunkData,
                 for (int j = 0; j < rightBuffer.count(); j++) {
                     const TextLineData &lineData = rightBuffer.at(j);
                     const QString line = DiffUtils::makePatchLine(
-                                QLatin1Char('+'),
+                                '+',
                                 lineData.text,
                                 lastChunk,
                                 i == chunkData.rows.count()
                                 && j == rightBuffer.count() - 1);
 
-                    const int blockDelta = line.count(QLatin1Char('\n')); // no new line
+                    const int blockDelta = line.count('\n'); // no new line
                                                      // could have been added
 
                     for (int k = 0; k < blockDelta; k++)
                         (*selections)[*blockNumber + blockCount + 1 + k].append(&m_controller.m_rightLineFormat);
-                    QMapIterator<int, int> itPos(lineData.changedPositions);
-                    while (itPos.hasNext()) {
-                        itPos.next();
-                        const int startPos = itPos.key() < 0
-                                ? 1 : itPos.key() + 1;
-                        const int endPos = itPos.value() < 0
-                                ? itPos.value() : itPos.value() + 1;
+
+                    for (auto it = lineData.changedPositions.cbegin(),
+                              end = lineData.changedPositions.cend(); it != end; ++it) {
+                        const int startPos = it.key() < 0
+                                ? 1 : it.key() + 1;
+                        const int endPos = it.value() < 0
+                                ? it.value() : it.value() + 1;
                         (*selections)[*blockNumber + blockCount + 1].append
                                 (DiffSelection(startPos, endPos, &m_controller.m_rightCharFormat));
                     }
@@ -377,7 +405,7 @@ QString UnifiedDiffEditorWidget::showChunk(const ChunkData &chunkData,
                 rightBuffer.clear();
             }
             if (i < chunkData.rows.count()) {
-                const QString line = DiffUtils::makePatchLine(QLatin1Char(' '),
+                const QString line = DiffUtils::makePatchLine(' ',
                                           rowData.rightLine.text,
                                           lastChunk,
                                           i == chunkData.rows.count() - 1);
@@ -389,7 +417,7 @@ QString UnifiedDiffEditorWidget::showChunk(const ChunkData &chunkData,
                     setRightLineNumber(*blockNumber + blockCount + 1,
                                        chunkData.rightStartingLineNumber
                                        + rightLineCount + 1);
-                    blockCount += line.count(QLatin1Char('\n'));
+                    blockCount += line.count('\n');
                     ++leftLineCount;
                     ++rightLineCount;
                 }
@@ -406,17 +434,17 @@ QString UnifiedDiffEditorWidget::showChunk(const ChunkData &chunkData,
         }
     }
 
-    const QString chunkLine = QLatin1String("@@ -")
+    const QString chunkLine = "@@ -"
             + QString::number(chunkData.leftStartingLineNumber + 1)
-            + QLatin1Char(',')
+            + ','
             + QString::number(leftLineCount)
-            + QLatin1String(" +")
+            + " +"
             + QString::number(chunkData.rightStartingLineNumber+ 1)
-            + QLatin1Char(',')
+            + ','
             + QString::number(rightLineCount)
-            + QLatin1String(" @@")
+            + " @@"
             + chunkData.contextInfo
-            + QLatin1Char('\n');
+            + '\n';
 
     diffText.prepend(chunkLine);
 
@@ -432,17 +460,21 @@ void UnifiedDiffEditorWidget::showDiff()
     int blockNumber = 0;
     int charNumber = 0;
 
+    // 'foldingIndent' is populated with <block number> and folding indentation
+    // value where 1 indicates start of new file and 2 indicates a diff chunk.
+    // Remaining lines (diff contents) are assigned 3.
+    QHash<int, int> foldingIndent;
+
     QMap<int, QList<DiffSelection> > selections;
 
-    for (int i = 0; i < m_controller.m_contextFileData.count(); i++) {
-        const FileData &fileData = m_controller.m_contextFileData.at(i);
-        const QString leftFileInfo = QLatin1String("--- ")
-                + fileData.leftFileInfo.fileName + QLatin1Char('\n');
-        const QString rightFileInfo = QLatin1String("+++ ")
-                + fileData.rightFileInfo.fileName + QLatin1Char('\n');
+    for (const FileData &fileData : m_controller.m_contextFileData) {
+        const QString leftFileInfo = "--- " + fileData.leftFileInfo.fileName + '\n';
+        const QString rightFileInfo = "+++ " + fileData.rightFileInfo.fileName + '\n';
         setFileInfo(blockNumber, fileData.leftFileInfo, fileData.rightFileInfo);
+        foldingIndent.insert(blockNumber, 1);
         selections[blockNumber].append(DiffSelection(&m_controller.m_fileLineFormat));
         blockNumber++;
+        foldingIndent.insert(blockNumber, 1);
         selections[blockNumber].append(DiffSelection(&m_controller.m_fileLineFormat));
         blockNumber++;
 
@@ -451,18 +483,20 @@ void UnifiedDiffEditorWidget::showDiff()
         charNumber += leftFileInfo.count() + rightFileInfo.count();
 
         if (fileData.binaryFiles) {
+            foldingIndent.insert(blockNumber, 2);
             selections[blockNumber].append(DiffSelection(&m_controller.m_chunkLineFormat));
             blockNumber++;
-            const QString binaryLine = QLatin1String("Binary files ")
+            const QString binaryLine = "Binary files "
                     + fileData.leftFileInfo.fileName
-                    + QLatin1String(" and ")
+                    + " and "
                     + fileData.rightFileInfo.fileName
-                    + QLatin1String(" differ\n");
+                    + " differ\n";
             diffText += binaryLine;
             charNumber += binaryLine.count();
         } else {
             for (int j = 0; j < fileData.chunks.count(); j++) {
                 const int oldBlockNumber = blockNumber;
+                foldingIndent.insert(blockNumber, 2);
                 diffText += showChunk(fileData.chunks.at(j),
                                       (j == fileData.chunks.count() - 1)
                                       && fileData.lastChunkAtTheEndOfFile,
@@ -481,10 +515,15 @@ void UnifiedDiffEditorWidget::showDiff()
         return;
     }
 
-    diffText.replace(QLatin1Char('\r'), QLatin1Char(' '));
+    diffText.replace('\r', ' ');
     const bool oldIgnore = m_controller.m_ignoreCurrentIndexChange;
     m_controller.m_ignoreCurrentIndexChange = true;
     setPlainText(diffText);
+
+    QTextBlock block = document()->firstBlock();
+    for (int b = 0; block.isValid(); block = block.next(), ++b)
+        setFoldingIndent(block, foldingIndent.value(b, 3));
+
     m_controller.m_ignoreCurrentIndexChange = oldIgnore;
 
     setSelections(selections);
@@ -495,28 +534,17 @@ int UnifiedDiffEditorWidget::blockNumberForFileIndex(int fileIndex) const
     if (fileIndex < 0 || fileIndex >= m_fileInfo.count())
         return -1;
 
-    QMap<int, QPair<DiffFileInfo, DiffFileInfo> >::const_iterator it
-            = m_fileInfo.constBegin();
-    for (int i = 0; i < fileIndex; i++)
-        ++it;
-
-    return it.key();
+    return (m_fileInfo.constBegin() + fileIndex).key();
 }
 
 int UnifiedDiffEditorWidget::fileIndexForBlockNumber(int blockNumber) const
 {
-    QMap<int, QPair<DiffFileInfo, DiffFileInfo> >::const_iterator it
-            = m_fileInfo.constBegin();
-    QMap<int, QPair<DiffFileInfo, DiffFileInfo> >::const_iterator itEnd
-            = m_fileInfo.constEnd();
-
     int i = -1;
-    while (it != itEnd) {
+    for (auto it = m_fileInfo.cbegin(), end = m_fileInfo.cend(); it != end; ++it, ++i) {
         if (it.key() > blockNumber)
             break;
-        ++it;
-        ++i;
     }
+
     return i;
 }
 
@@ -525,8 +553,7 @@ int UnifiedDiffEditorWidget::chunkIndexForBlockNumber(int blockNumber) const
     if (m_chunkInfo.isEmpty())
         return -1;
 
-    QMap<int, QPair<int, int> >::const_iterator it
-            = m_chunkInfo.upperBound(blockNumber);
+    auto it = m_chunkInfo.upperBound(blockNumber);
     if (it == m_chunkInfo.constBegin())
         return -1;
 
@@ -563,14 +590,12 @@ void UnifiedDiffEditorWidget::jumpToOriginalFile(const QTextCursor &cursor)
     const int leftLineNumber = m_leftLineNumbers.value(blockNumber, -1);
     if (leftLineNumber >= 0) {
         if (leftFileName == rightFileName) {
-            for (int i = 0; i < fileData.chunks.count(); i++) {
-                const ChunkData chunkData = fileData.chunks.at(i);
+            for (const ChunkData &chunkData : fileData.chunks) {
 
                 int newLeftLineNumber = chunkData.leftStartingLineNumber;
                 int newRightLineNumber = chunkData.rightStartingLineNumber;
 
-                for (int j = 0; j < chunkData.rows.count(); j++) {
-                    const RowData rowData = chunkData.rows.at(j);
+                for (const RowData &rowData : chunkData.rows) {
                     if (rowData.leftLine.textLineType == TextLineData::TextLine)
                         newLeftLineNumber++;
                     if (rowData.rightLine.textLineType == TextLineData::TextLine)
