@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -83,45 +78,86 @@ QString QmakeBuildConfiguration::shadowBuildDirectory(const QString &proFilePath
         return QString();
 
     const QString projectName = QFileInfo(proFilePath).completeBaseName();
-    ProjectMacroExpander expander(projectName, k, suffix, buildType);
+    ProjectMacroExpander expander(proFilePath, projectName, k, suffix, buildType);
     QString projectDir = Project::projectDirectory(FileName::fromString(proFilePath)).toString();
     QString buildPath = expander.expand(Core::DocumentManager::buildDirectory());
     return FileUtils::resolvePath(projectDir, buildPath);
 }
 
-static Utils::FileName defaultBuildDirectory(const QString &projectPath,
-                                             const ProjectExplorer::Kit *k,
-                                             const QString &suffix,
-                                             BuildConfiguration::BuildType type)
+static FileName defaultBuildDirectory(const QString &projectPath,
+                                      const Kit *k,
+                                      const QString &suffix,
+                                      BuildConfiguration::BuildType type)
 {
-    return Utils::FileName::fromString(QmakeBuildConfiguration::shadowBuildDirectory(projectPath, k,
-                                                                                     suffix, type));
+    return FileName::fromString(QmakeBuildConfiguration::shadowBuildDirectory(projectPath, k,
+                                                                              suffix, type));
 }
 
-const char QMAKE_BC_ID[] = "Qt4ProjectManager.Qt4BuildConfiguration";
 const char USE_SHADOW_BUILD_KEY[] = "Qt4ProjectManager.Qt4BuildConfiguration.UseShadowBuild";
 const char BUILD_CONFIGURATION_KEY[] = "Qt4ProjectManager.Qt4BuildConfiguration.BuildConfiguration";
 
 enum { debug = 0 };
 
 QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target)
-    : QmakeBuildConfiguration(target, Core::Id(QMAKE_BC_ID))
+    : BuildConfiguration(target, Constants::QMAKE_BC_ID)
 {
+    connect(this, &BuildConfiguration::buildDirectoryChanged,
+            this, &QmakeBuildConfiguration::emitProFileEvaluateNeeded);
+    connect(this, &BuildConfiguration::environmentChanged,
+            this, &QmakeBuildConfiguration::emitProFileEvaluateNeeded);
+    connect(target, &Target::kitChanged,
+            this, &QmakeBuildConfiguration::kitChanged);
+    MacroExpander *expander = macroExpander();
+    expander->registerVariable("Qmake:Makefile", "Qmake makefile", [this]() -> QString {
+        const QString file = makefile();
+        if (!file.isEmpty())
+            return file;
+        return QLatin1String("Makefile");
+    });
+    connect(ToolChainManager::instance(), &ToolChainManager::toolChainUpdated,
+            this, &QmakeBuildConfiguration::toolChainUpdated);
+    connect(QtVersionManager::instance(), &QtVersionManager::qtVersionsChanged,
+            this, &QmakeBuildConfiguration::qtVersionsChanged);
 }
 
-QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, Core::Id id) :
-    BuildConfiguration(target, id)
+void QmakeBuildConfiguration::initialize(const BuildInfo *info)
 {
-    ctor();
-}
+    BuildConfiguration::initialize(info);
 
-QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, QmakeBuildConfiguration *source) :
-    BuildConfiguration(target, source),
-    m_shadowBuild(source->m_shadowBuild),
-    m_qmakeBuildConfiguration(source->m_qmakeBuildConfiguration)
-{
-    cloneSteps(source);
-    ctor();
+    BuildStepList *buildSteps = stepList(ProjectExplorer::Constants::BUILDSTEPS_BUILD);
+    auto qmakeStep = new QMakeStep(buildSteps);
+    buildSteps->appendStep(qmakeStep);
+    buildSteps->appendStep(new MakeStep(buildSteps));
+
+    BuildStepList *cleanSteps = stepList(ProjectExplorer::Constants::BUILDSTEPS_CLEAN);
+    cleanSteps->appendStep(new MakeStep(cleanSteps));
+
+    const QmakeBuildInfo *qmakeInfo = static_cast<const QmakeBuildInfo *>(info);
+    BaseQtVersion *version = QtKitInformation::qtVersion(target()->kit());
+
+    BaseQtVersion::QmakeBuildConfigs config = version->defaultBuildConfig();
+    if (qmakeInfo->buildType == BuildConfiguration::Debug)
+        config |= BaseQtVersion::DebugBuild;
+    else
+        config &= ~BaseQtVersion::DebugBuild;
+
+    QString additionalArguments = qmakeInfo->additionalArguments;
+    if (!additionalArguments.isEmpty())
+        qmakeStep->setUserArguments(additionalArguments);
+    qmakeStep->setLinkQmlDebuggingLibrary(qmakeInfo->config.linkQmlDebuggingQQ2);
+    qmakeStep->setSeparateDebugInfo(qmakeInfo->config.separateDebugInfo);
+    qmakeStep->setUseQtQuickCompiler(qmakeInfo->config.useQtQuickCompiler);
+
+    setQMakeBuildConfiguration(config);
+
+    FileName directory = qmakeInfo->buildDirectory;
+    if (directory.isEmpty()) {
+        directory = defaultBuildDirectory(target()->project()->projectFilePath().toString(),
+                                          target()->kit(), qmakeInfo->displayName, buildType());
+    }
+
+    setBuildDirectory(directory);
+    updateCacheAndEmitEnvironmentChanged();
 }
 
 QmakeBuildConfiguration::~QmakeBuildConfiguration()
@@ -145,22 +181,9 @@ bool QmakeBuildConfiguration::fromMap(const QVariantMap &map)
     m_qmakeBuildConfiguration = BaseQtVersion::QmakeBuildConfigs(map.value(QLatin1String(BUILD_CONFIGURATION_KEY)).toInt());
 
     m_lastKitState = LastKitState(target()->kit());
-
-    connect(ProjectExplorer::ToolChainManager::instance(), SIGNAL(toolChainUpdated(ProjectExplorer::ToolChain*)),
-            this, SLOT(toolChainUpdated(ProjectExplorer::ToolChain*)));
-    connect(QtSupport::QtVersionManager::instance(), SIGNAL(qtVersionsChanged(QList<int>,QList<int>,QList<int>)),
-            this, SLOT(qtVersionsChanged(QList<int>,QList<int>,QList<int>)));
     return true;
 }
 
-void QmakeBuildConfiguration::ctor()
-{
-    connect(this, SIGNAL(environmentChanged()),
-            this, SLOT(emitProFileEvaluateNeeded()));
-    connect(target(), SIGNAL(kitChanged()),
-            this, SLOT(kitChanged()));
-}
-
 void QmakeBuildConfiguration::kitChanged()
 {
     LastKitState newState = LastKitState(target()->kit());
@@ -173,9 +196,9 @@ void QmakeBuildConfiguration::kitChanged()
     }
 }
 
-void QmakeBuildConfiguration::toolChainUpdated(ProjectExplorer::ToolChain *tc)
+void QmakeBuildConfiguration::toolChainUpdated(ToolChain *tc)
 {
-    if (ToolChainKitInformation::toolChain(target()->kit()) == tc)
+    if (ToolChainKitInformation::toolChain(target()->kit(), ProjectExplorer::Constants::CXX_LANGUAGE_ID) == tc)
         emitProFileEvaluateNeeded();
 }
 
@@ -228,17 +251,9 @@ bool QmakeBuildConfiguration::isShadowBuild() const
     return buildDirectory() != target()->project()->projectDirectory();
 }
 
-void QmakeBuildConfiguration::setBuildDirectory(const FileName &directory)
-{
-    if (directory == buildDirectory())
-        return;
-    BuildConfiguration::setBuildDirectory(directory);
-    emitProFileEvaluateNeeded();
-}
-
 QString QmakeBuildConfiguration::makefile() const
 {
-    return static_cast<QmakeProject *>(target()->project())->rootQmakeProjectNode()->makefile();
+    return static_cast<QmakeProject *>(target()->project())->rootProFile()->makefile();
 }
 
 BaseQtVersion::QmakeBuildConfigs QmakeBuildConfiguration::qmakeBuildConfiguration() const
@@ -476,7 +491,7 @@ FileName QmakeBuildConfiguration::extractSpecFromArguments(QString *args,
 
     FileName baseMkspecDir = FileName::fromUserInput(
             version->qmakeProperty("QT_HOST_DATA") + QLatin1String("/mkspecs"));
-    baseMkspecDir = Utils::FileName::fromString(baseMkspecDir.toFileInfo().canonicalFilePath());
+    baseMkspecDir = FileName::fromString(baseMkspecDir.toFileInfo().canonicalFilePath());
 
     // if the path is relative it can be
     // relative to the working directory (as found in the Makefiles)
@@ -531,37 +546,18 @@ void QmakeBuildConfiguration::setEnabled(bool enabled)
   \class QmakeBuildConfigurationFactory
 */
 
-QmakeBuildConfigurationFactory::QmakeBuildConfigurationFactory(QObject *parent) :
-    IBuildConfigurationFactory(parent)
+QmakeBuildConfigurationFactory::QmakeBuildConfigurationFactory()
 {
-    update();
-
-    connect(QtVersionManager::instance(),
-            SIGNAL(qtVersionsChanged(QList<int>,QList<int>,QList<int>)),
-            this, SLOT(update()));
-}
-
-QmakeBuildConfigurationFactory::~QmakeBuildConfigurationFactory()
-{
-}
-
-void QmakeBuildConfigurationFactory::update()
-{
-    emit availableCreationIdsChanged();
-}
-
-bool QmakeBuildConfigurationFactory::canHandle(const Target *t) const
-{
-    if (!t->project()->supportsKit(t->kit()))
-        return false;
-    return qobject_cast<QmakeProject *>(t->project());
+    registerBuildConfiguration<QmakeBuildConfiguration>(Constants::QMAKE_BC_ID);
+    setSupportedProjectType(Constants::QMAKEPROJECT_ID);
+    setSupportedProjectMimeTypeName(Constants::PROFILE_MIMETYPE);
 }
 
 QmakeBuildInfo *QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
                                                               const QString &projectPath,
                                                               BuildConfiguration::BuildType type) const
 {
-    QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(k);
+    BaseQtVersion *version = QtKitInformation::qtVersion(k);
     QmakeBuildInfo *info = new QmakeBuildInfo(this);
     QString suffix;
     if (type == BuildConfiguration::Release) {
@@ -569,6 +565,8 @@ QmakeBuildInfo *QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
         info->displayName = tr("Release");
         //: Non-ASCII characters in directory suffix may cause build issues.
         suffix = tr("Release", "Shadow build directory suffix");
+        if (version && version->isQtQuickCompilerSupported())
+            info->config.useQtQuickCompiler = true;
     } else {
         if (type == BuildConfiguration::Debug) {
             //: The name of the debug build configuration created by default for a qmake project.
@@ -581,8 +579,10 @@ QmakeBuildInfo *QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
             //: Non-ASCII characters in directory suffix may cause build issues.
             suffix = tr("Profile", "Shadow build directory suffix");
             info->config.separateDebugInfo = true;
+            if (version && version->isQtQuickCompilerSupported())
+                info->config.useQtQuickCompiler = true;
         }
-        if (version && version->qtVersion().majorVersion >= 5)
+        if (version && version->isQmlDebuggingSupported())
             info->config.linkQmlDebuggingQQ2 = true;
     }
     info->typeName = info->displayName;
@@ -590,16 +590,16 @@ QmakeBuildInfo *QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
     info->kitId = k->id();
 
     // check if this project is in the source directory:
-    Utils::FileName projectFilePath = Utils::FileName::fromString(projectPath);
+    FileName projectFilePath = FileName::fromString(projectPath);
     if (version && version->isInSourceDirectory(projectFilePath)) {
         // assemble build directory
         QString projectDirectory = projectFilePath.toFileInfo().absolutePath();
         QDir qtSourceDir = QDir(version->sourcePath().toString());
         QString relativeProjectPath = qtSourceDir.relativeFilePath(projectDirectory);
-        QString qtBuildDir = version->versionInfo().value(QStringLiteral("QT_INSTALL_PREFIX"));
+        QString qtBuildDir = version->qmakeProperty("QT_INSTALL_PREFIX");
         QString absoluteBuildPath = QDir::cleanPath(qtBuildDir + QLatin1Char('/') + relativeProjectPath);
 
-        info->buildDirectory = Utils::FileName::fromString(absoluteBuildPath);
+        info->buildDirectory = FileName::fromString(absoluteBuildPath);
     } else {
         info->buildDirectory = defaultBuildDirectory(projectPath, k, suffix, type);
     }
@@ -607,11 +607,6 @@ QmakeBuildInfo *QmakeBuildConfigurationFactory::createBuildInfo(const Kit *k,
     return info;
 }
 
-int QmakeBuildConfigurationFactory::priority(const Target *parent) const
-{
-    return canHandle(parent) ? 0 : -1;
-}
-
 static QList<BuildConfiguration::BuildType> availableBuildTypes(const BaseQtVersion *version)
 {
     QList<BuildConfiguration::BuildType> types = {BuildConfiguration::Debug,
@@ -623,12 +618,12 @@ static QList<BuildConfiguration::BuildType> availableBuildTypes(const BaseQtVers
 
 QList<BuildInfo *> QmakeBuildConfigurationFactory::availableBuilds(const Target *parent) const
 {
-    QList<ProjectExplorer::BuildInfo *> result;
+    QList<BuildInfo *> result;
 
     const QString projectFilePath = parent->project()->projectFilePath().toString();
 
     foreach (BuildConfiguration::BuildType buildType,
-             availableBuildTypes(QtSupport::QtKitInformation::qtVersion(parent->kit()))) {
+             availableBuildTypes(QtKitInformation::qtVersion(parent->kit()))) {
         QmakeBuildInfo *info = createBuildInfo(parent->kit(), projectFilePath, buildType);
         info->displayName.clear(); // ask for a name
         info->buildDirectory.clear(); // This depends on the displayName
@@ -638,18 +633,10 @@ QList<BuildInfo *> QmakeBuildConfigurationFactory::availableBuilds(const Target
     return result;
 }
 
-int QmakeBuildConfigurationFactory::priority(const Kit *k, const QString &projectPath) const
-{
-    Utils::MimeDatabase mdb;
-    if (k && mdb.mimeTypeForFile(projectPath).matchesName(QLatin1String(Constants::PROFILE_MIMETYPE)))
-        return 0;
-    return -1;
-}
-
 QList<BuildInfo *> QmakeBuildConfigurationFactory::availableSetups(const Kit *k, const QString &projectPath) const
 {
-    QList<ProjectExplorer::BuildInfo *> result;
-    QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(k);
+    QList<BuildInfo *> result;
+    BaseQtVersion *qtVersion = QtKitInformation::qtVersion(k);
     if (!qtVersion || !qtVersion->isValid())
         return result;
 
@@ -659,98 +646,6 @@ QList<BuildInfo *> QmakeBuildConfigurationFactory::availableSetups(const Kit *k,
     return result;
 }
 
-void QmakeBuildConfigurationFactory::configureBuildConfiguration(Target *parent,
-                                                                 QmakeBuildConfiguration *bc,
-                                                                 const QmakeBuildInfo *qmakeInfo) const
-{
-    BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(parent->kit());
-
-    BaseQtVersion::QmakeBuildConfigs config = version->defaultBuildConfig();
-    if (qmakeInfo->buildType == BuildConfiguration::Debug)
-        config |= QtSupport::BaseQtVersion::DebugBuild;
-    else
-        config &= ~QtSupport::BaseQtVersion::DebugBuild;
-
-    bc->setDefaultDisplayName(qmakeInfo->displayName);
-    bc->setDisplayName(qmakeInfo->displayName);
-
-    BuildStepList *buildSteps = bc->stepList(Core::Id(ProjectExplorer::Constants::BUILDSTEPS_BUILD));
-    BuildStepList *cleanSteps = bc->stepList(Core::Id(ProjectExplorer::Constants::BUILDSTEPS_CLEAN));
-    Q_ASSERT(buildSteps);
-    Q_ASSERT(cleanSteps);
-
-    QMakeStep *qmakeStep = new QMakeStep(buildSteps);
-    buildSteps->insertStep(0, qmakeStep);
-
-    MakeStep *makeStep = new MakeStep(buildSteps);
-    buildSteps->insertStep(1, makeStep);
-
-    MakeStep *cleanStep = new MakeStep(cleanSteps);
-    cleanStep->setClean(true);
-    cleanStep->setUserArguments(QLatin1String("clean"));
-    cleanSteps->insertStep(0, cleanStep);
-
-    QString additionalArguments = qmakeInfo->additionalArguments;
-    if (!additionalArguments.isEmpty())
-        qmakeStep->setUserArguments(additionalArguments);
-    qmakeStep->setLinkQmlDebuggingLibrary(qmakeInfo->config.linkQmlDebuggingQQ2);
-    qmakeStep->setSeparateDebugInfo(qmakeInfo->config.separateDebugInfo);
-    qmakeStep->setUseQtQuickCompiler(qmakeInfo->config.useQtQuickCompiler);
-
-    bc->setQMakeBuildConfiguration(config);
-
-    Utils::FileName directory = qmakeInfo->buildDirectory;
-    if (directory.isEmpty()) {
-        directory = defaultBuildDirectory(parent->project()->projectFilePath().toString(),
-                                          parent->kit(), qmakeInfo->displayName, bc->buildType());
-    }
-
-    bc->setBuildDirectory(directory);
-}
-
-BuildConfiguration *QmakeBuildConfigurationFactory::create(Target *parent, const BuildInfo *info) const
-{
-    QTC_ASSERT(info->factory() == this, return 0);
-    QTC_ASSERT(info->kitId == parent->kit()->id(), return 0);
-    QTC_ASSERT(!info->displayName.isEmpty(), return 0);
-
-    const QmakeBuildInfo *qmakeInfo = static_cast<const QmakeBuildInfo *>(info);
-    QmakeBuildConfiguration *bc = new QmakeBuildConfiguration(parent);
-    configureBuildConfiguration(parent, bc, qmakeInfo);
-    return bc;
-}
-
-bool QmakeBuildConfigurationFactory::canClone(const Target *parent, BuildConfiguration *source) const
-{
-    return canHandle(parent) && qobject_cast<QmakeBuildConfiguration *>(source);
-}
-
-BuildConfiguration *QmakeBuildConfigurationFactory::clone(Target *parent, BuildConfiguration *source)
-{
-    if (!canClone(parent, source))
-        return 0;
-    QmakeBuildConfiguration *oldbc(static_cast<QmakeBuildConfiguration *>(source));
-    return new QmakeBuildConfiguration(parent, oldbc);
-}
-
-bool QmakeBuildConfigurationFactory::canRestore(const Target *parent, const QVariantMap &map) const
-{
-    if (!canHandle(parent))
-        return false;
-    return ProjectExplorer::idFromMap(map) == QMAKE_BC_ID;
-}
-
-BuildConfiguration *QmakeBuildConfigurationFactory::restore(Target *parent, const QVariantMap &map)
-{
-    if (!canRestore(parent, map))
-        return 0;
-    QmakeBuildConfiguration *bc = new QmakeBuildConfiguration(parent);
-    if (bc->fromMap(map))
-        return bc;
-    delete bc;
-    return 0;
-}
-
 BuildConfiguration::BuildType QmakeBuildConfiguration::buildType() const
 {
     QMakeStep *qs = qmakeStep();
@@ -762,17 +657,27 @@ BuildConfiguration::BuildType QmakeBuildConfiguration::buildType() const
         return Release;
 }
 
-QmakeBuildConfiguration::LastKitState::LastKitState()
+void QmakeBuildConfiguration::addToEnvironment(Environment &env) const
 {
-
+    setupBuildEnvironment(target()->kit(), env);
 }
 
+void QmakeBuildConfiguration::setupBuildEnvironment(Kit *k, Environment &env)
+{
+    prependCompilerPathToEnvironment(k, env);
+    const BaseQtVersion *qt = QtKitInformation::qtVersion(k);
+    if (qt && !qt->binPath().isEmpty())
+        env.prependOrSetPath(qt->binPath().toString());
+}
+
+QmakeBuildConfiguration::LastKitState::LastKitState() { }
+
 QmakeBuildConfiguration::LastKitState::LastKitState(Kit *k)
     : m_qtVersion(QtKitInformation::qtVersionId(k)),
       m_sysroot(SysRootKitInformation::sysRoot(k).toString()),
       m_mkspec(QmakeKitInformation::mkspec(k).toString())
 {
-    ToolChain *tc = ToolChainKitInformation::toolChain(k);
+    ToolChain *tc = ToolChainKitInformation::toolChain(k, ProjectExplorer::Constants::CXX_LANGUAGE_ID);
     m_toolchain = tc ? tc->id() : QByteArray();
 }
 
